package fr.natan.employeemicroservice.service.services;

import fr.natan.employeemicroservice.entity.Employee;
import fr.natan.employeemicroservice.exceptions.AddressNotFoundException;
import fr.natan.employeemicroservice.exceptions.EmployeeAlreadyExistsException;
import fr.natan.employeemicroservice.exceptions.EmployeeNotFoundException;
import fr.natan.employeemicroservice.exceptions.EmployeeOneOrMoreFieldsEmptyException;
import fr.natan.employeemicroservice.repository.EmployeeRepository;
import fr.natan.employeemicroservice.service.contract.EmployeeContract;
import fr.natan.employeemicroservice.t_openFeign.address.Address;
import fr.natan.employeemicroservice.t_openFeign.openFeign.AddressServiceOpenFeign;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Time;
import java.util.List;

@SuppressWarnings({"FeatureEnvy", "RedundantIfStatement", "ConstantConditions", "PointlessBooleanExpression"})
@Service
public class EmployeeService implements EmployeeContract {
    private final EmployeeRepository employeeRepository;
    private final AddressServiceOpenFeign addressServiceOpenFeign;

    public EmployeeService(EmployeeRepository employeeRepository, AddressServiceOpenFeign addressServiceOpenFeign) {
        this.employeeRepository = employeeRepository;
        this.addressServiceOpenFeign = addressServiceOpenFeign;
    }

    private Employee normalizeEmployee(Employee employee){
        employee.setFirstname(employee.getFirstname().strip());
        employee.setLastname(employee.getLastname().strip());
        employee.setBirthCountry(employee.getBirthCountry().strip());
        employee.setLastname(employee.getLastname().toUpperCase());

        return employee;
    }
    private boolean isValidEmployee(Employee employee){
        if(employee.getFirstname().isEmpty() || employee.getLastname().isEmpty()){
            return false;
        }
        else {
            return true;
        }
    }

    private String natanEmailBuilder(String firstname, String lastname){
        firstname = firstname.replaceAll("\\s","");
        lastname = lastname.replaceAll("\\s","");
        return firstname.toLowerCase()+"."+lastname.toLowerCase()+"@natan.fr";
    }
    @Override
    public Employee createEmployee(Employee employee, Long addressID) throws EmployeeAlreadyExistsException,
            EmployeeOneOrMoreFieldsEmptyException, AddressNotFoundException {

        employee = normalizeEmployee(employee);
        if(isValidEmployee(employee)==false){
            throw new EmployeeOneOrMoreFieldsEmptyException("one or more fields are empty");
        }
        Address address  = addressServiceOpenFeign.findAddressById(addressID);
        if(address==null){
            throw new AddressNotFoundException("Address to affect new employee is not found");
        }
        Employee saveToEmployee = getEmployeeByName(employee.getFirstname(), employee.getLastname());
        if(saveToEmployee!= null){
            throw new EmployeeAlreadyExistsException("employee already exists");
        }
        employee.setEmail(
                natanEmailBuilder(
                        employee.getFirstname(), employee.getLastname()
                )
        );
        employee.setAddressID(addressID);
        employee.setAddress(address);
        employee.setCreatedDate(new Date(System.currentTimeMillis()) +" : "+ new Time(System.currentTimeMillis()));

        return employeeRepository.save(employee);
    }

    @Override
    public Employee getEmployeeByName(String firstname, String lastname) {
        return employeeRepository.findByFirstnameAndLastname(firstname, lastname);
    }

    @Override
    public List<Employee> getAllEmployees() {
        return employeeRepository.findAll();
    }

    @Override
    public Employee getEmployeeByID(Long employeeID) throws EmployeeNotFoundException {
        return employeeRepository.findById(employeeID).orElseThrow(
                () -> new EmployeeNotFoundException("employee not found")
        );
    }

    @Override
    public List<Employee> getEmployeesLivingAtAddressID(Long addressID) throws AddressNotFoundException {

        Address address = addressServiceOpenFeign.findAddressById(addressID);
        if(address == null){
            throw new AddressNotFoundException("address not found");
        }
        return employeeRepository.findByAddressID(addressID);
    }

    @Override
    public Employee updateEmployee(Long employeeID, Employee employee) throws EmployeeNotFoundException, AddressNotFoundException {
      getEmployeeByID(employeeID);
      //to update employee address check first the new address exists
        Address address = addressServiceOpenFeign.findAddressById(employee.getAddressID());
        if(address==null){
            throw new AddressNotFoundException("address to re-affect employee not found");
        }
      return employeeRepository.save(employee);
    }

    @Override
    public void deleteEmployee(Long employeeID) throws EmployeeNotFoundException {
        Employee employee = getEmployeeByID(employeeID);
        employeeRepository.delete(employee);

    }

}
