package fr.natan.projectmicroservice.service.service;

import fr.natan.projectmicroservice.exceptions.*;
import fr.natan.projectmicroservice.model.entity.Project;
import fr.natan.projectmicroservice.repository.ProjectRepository;
import fr.natan.projectmicroservice.service.contract.ProjectContract;
import fr.natan.projectmicroservice.t_openFeign.company_model.model.Company;
import fr.natan.projectmicroservice.t_openFeign.employee_model.Employee;
import fr.natan.projectmicroservice.t_openFeign.openFeign.CompanyServiceOpenFeign;
import fr.natan.projectmicroservice.t_openFeign.openFeign.EmployeeServiceOpenFeign;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Time;
import java.util.List;

@Service
public class ProjectService implements ProjectContract {

    private final ProjectRepository projectRepository;
    private final EmployeeServiceOpenFeign employeeServiceOpenFeign;
    private final CompanyServiceOpenFeign companyServiceOpenFeign;

    public ProjectService(
            ProjectRepository projectRepository,
            EmployeeServiceOpenFeign employeeServiceOpenFeign,
            CompanyServiceOpenFeign companyServiceOpenFeign) {
        this.projectRepository = projectRepository;
        this.employeeServiceOpenFeign = employeeServiceOpenFeign;
        this.companyServiceOpenFeign = companyServiceOpenFeign;
    }

    @Override
    public Project getProjectByName(String projectName) {
        return projectRepository.findByProjectName(projectName);
    }

    private Project normalize(Project project){
        project.setProjectName(project.getProjectName().strip().toUpperCase());
        project.setDescription(project.getDescription().strip());

        return project;
    }

    private boolean isInvalidProject(Project project){
        if(project.getProjectName().isEmpty()
                || project.getDescription().isEmpty()){
            return true;
        }
        else
            return false;
    }

    private boolean isInvalidProjectDescription(String projectDescription){
        if(projectDescription.length()<=25)
            return true;
        else
            return false;
    }

    private Employee getEmployee(Long employeeID) throws EmployeeNotFoundException {
        Employee employee = employeeServiceOpenFeign.getEmployeeByID(employeeID);
        if(employee==null){
            throw new EmployeeNotFoundException("employee not found");
        }

        return employee;
    }

    private Company getCompany(Long companyID) throws CompanyNotFoundException {
        Company company = companyServiceOpenFeign.getCompanyByID(companyID);
        if (company==null) {
            throw new CompanyNotFoundException("company not found");
        }

        return company;
    }
    @Override
    public Project createProject(Project project, Long employeeID, Long companyID)
            throws ProjectAlreadyExistsException, ProjectOneOrMoreFieldsInvalidException, EmployeeNotFoundException,
            CompanyNotFoundException, ProjectDescriptionShortException {

        project = normalize(project);
        if(isInvalidProject(project)){
            throw  new ProjectOneOrMoreFieldsInvalidException("one or more fields invalid");
        } else if (isInvalidProjectDescription(project.getDescription())) {
            throw new ProjectDescriptionShortException("short project description");
        }
        //test if employee and company to which project is affected exist
        Employee employee = getEmployee(employeeID);
        Company company = getCompany(companyID);

        Project savedProject = getProjectByName(project.getProjectName());
        if(savedProject!=null){
            throw new ProjectAlreadyExistsException("project already exists");
        }
        project.setCreatedDate(new Date(System.currentTimeMillis())+" : "+new Time(System.currentTimeMillis()));
        project.setEmployeeID(employeeID);
        project.setCompanyID(companyID);
        project.setEmployee(employee);
        project.setCompany(company);

        return projectRepository.save(project);
    }

    @Override
    public List<Project> getAllProjects() {
        return projectRepository.findAll();
    }

    @Override
    public Project getProjectByID(Long projectID) throws ProjectNotFoundException {
        return projectRepository.findById(projectID).orElseThrow(
                ()-> new ProjectNotFoundException("project not found")
        );
    }

    @Override
    public Project updateProject(Long projectID, Project project)
            throws ProjectNotFoundException, EmployeeNotFoundException, CompanyNotFoundException,
            ProjectOneOrMoreFieldsInvalidException, ProjectDescriptionShortException {
        //to update project, we need to test if the corresponding employee and company exists
        Employee savedEmployee = getEmployee(project.getEmployeeID());
        Company savedCompany = getCompany(project.getCompanyID());
        if(savedEmployee==null){
            throw new EmployeeNotFoundException("employee to re-affect project not found");
        } else if (savedCompany==null) {
            throw new CompanyNotFoundException("company to re-affect project not found");

        }

       //normalize project
        project = normalize(project);
        if(isInvalidProject(project)){
            throw new ProjectOneOrMoreFieldsInvalidException("ore or ore fields invalid");
        }
        else if (isInvalidProjectDescription(project.getDescription())) {
            throw new ProjectDescriptionShortException("short project description");
        }

        //verify if project to update exists
       getProjectByID(projectID);

        return projectRepository.save(project);
    }

    @Override
    public void deleteProject(Long projectID) throws ProjectNotFoundException {
        Project project = getProjectByID(projectID);// test if project to delete exists
        projectRepository.delete(project);
    }

    @Override
    public List<Project> getProjectsAffectedToEmployee(Long employeeID) throws EmployeeNotFoundException {
        getEmployee(employeeID);
        return projectRepository.findByEmployeeID(employeeID);
    }
    @Override
    public List<Project> getProjectsAffectedToCompany(Long companyID) throws CompanyNotFoundException {
        getCompany(companyID);
        return projectRepository.findByCompanyID(companyID);
    }

    @Override
    public List<Project> getProjectsAffectedToAEmployeeInACompany(Long employeeID, Long companyID)
            throws EmployeeNotFoundException, CompanyNotFoundException {
        getEmployee(employeeID);
        getCompany(companyID);

        return projectRepository.findByEmployeeIDAndCompanyID(employeeID, companyID);
    }
}
